package main

import (
	"flag"
	"fmt"
	"log"
	"os/exec"
	"strings"

	"github.com/songgao/packets/ethernet"
	"github.com/songgao/water"
	"github.com/songgao/water/waterutil"
)

func init() {
	log.SetFlags(log.Llongfile | log.LstdFlags)
}

//go:generate bash -c "protoc --go_out=plugins=grpc:. proto/*.proto"
func main() {
	testmain()
	// NewNetCard().Run()
}

var (
	flagAddr = flag.String("addr", "", "addr set")
)

func startCard() {
	cmdstr := fmt.Sprintf("ip addr add %s dev stun", *flagAddr)
	log.Println(cmdstr)
	cmd := strings.Split(cmdstr, " ")
	err := exec.Command(cmd[0], cmd[1:]...).Run()
	if err != nil {
		log.Panic(err)
	}

	cmdstr = "ip link set dev stun up"
	cmd = strings.Split(cmdstr, " ")
	err = exec.Command(cmd[0], cmd[1:]...).Run()
	if err != nil {
		log.Panic(err)
	}
}
func testmain() {

	flag.Parse()

	config := water.Config{
		DeviceType: water.TUN,
	}
	config.Name = "stun"

	ifce, err := water.New(config)
	if err != nil {
		log.Fatal(err)
	}

	startCard()

	var frame ethernet.Frame
	for {

		frame.Resize(1500)
		n, err := ifce.Read([]byte(frame))
		if err != nil {
			log.Fatal(err)
		}
		frame = frame[:n]

		if frame.Ethertype() != ethernet.ARP {
			log.Printf("Ethertype: % x\n", frame.Ethertype())

			log.Printf("DstMac: %s SrcMac: %s\n", waterutil.IPv4Destination(frame), waterutil.IPv4Source(frame))
			log.Printf("DstPort: %d SrcPort: %d\n", waterutil.IPv4DestinationPort(frame), waterutil.IPv4SourcePort(frame))
		}

		// log.Printf("Payload: % x\n", frame.Payload())

		ifce.Write(frame)
	}
}
